#include "PCPokktManager.h"
#include "PCParser.h"
#include "cjson.h"
#include <string>
#include <sstream>


EVT_SELECTOR_POKKT_INITIALIZED PCPokktManager::_pokktInitializedListner = NULL;
PCPokktInitializedDelegate* PCPokktManager::_pokktInitializedEventTarget = NULL;
std::map<std::string, PCPokktManager::ListenerTargetPairOfferwall*> PCPokktManager::_eventMapOfferwall;
std::map<std::string, PCPokktManager::ListenerTargetPairAdEvent*> PCPokktManager::_eventMapAdEvent;
bool PCPokktManager::_isReceiverAttatached = false;

std::string to_string(int value)
{
    std::stringstream temp_str;
    temp_str<<(value);
    std::string str = temp_str.str();
    return str;
}

void PCPokktManager::hookCallbackReceiver()
{
    if (!_isReceiverAttatached)
    {
        PokktNativeExtensionRegister(OPERATION_EVENT, (s3eCallback)PCPokktManager::pokktCallbackReceiver, NULL);
        _isReceiverAttatached = true;
    }
}

void PCPokktManager::unhookCallbackReceiver()
{
    PokktNativeExtensionUnRegister(OPERATION_EVENT, (s3eCallback)PCPokktManager::pokktCallbackReceiver);
    _isReceiverAttatached = false;
}

void PCPokktManager::pokktCallbackReceiver(const CallbackData* data, void* userData)
{
    //handleNativeMessage(data->operation, data->params);
    handleNativeMessage(data);
}

void PCPokktManager::initPokkt(PCPokktConfig* config)
{
    hookCallbackReceiver();
    notifyNative(INIT_POKKT_OP, toPokktConfigJSONString(config).c_str());
}

//void PCPokktManager::handleNativeMessage(const char* operation1, const char*  param1)
void PCPokktManager::handleNativeMessage(const CallbackData* data)
{
    std::string operation = data->operation;
    std::string param = data->params;
    
    // pokkt initialized event
    if (operation.compare(POKKT_INITIALISED_EVENT) == 0)
    {
        bool isReady = param.compare("true") == 0;
        (_pokktInitializedEventTarget->*_pokktInitializedListner)(isReady);
        return;
    }
    
    // filter offerwall events
    if ((operation.compare(COIN_RESPONSE_EVENT) == 0) ||
        (operation.compare(COIN_RESPONSE_WITH_TRANS_ID_EVENT) == 0) ||
        (operation.compare(COIN_RESPONSE_FAILED_EVENT) == 0) ||
        (operation.compare(CAMPAIGN_AVAILABILITY_EVENT) == 0) ||
        (operation.compare(OFFERWALL_CLOSED_EVENT) == 0))
    {
        ListenerTargetPairOfferwall *l = _eventMapOfferwall[operation];
        if (l != NULL)
        {
            EVT_SELECTOR_OFFERWALL listener = l->listener;
            PCPokktOfferwallDelegate* target = l->target;
            (target->*listener)(param);
        }
    }
    
    // filter ad-events
    if ((operation.compare(AD_CACHING_COMPLETED_EVENT) == 0) ||
        (operation.compare(AD_CACHING_FAILED_EVENT) == 0) ||
        (operation.compare(AD_CLOSED_EVENT) == 0) ||
        (operation.compare(AD_COMPLETED_EVENT) == 0) ||
        (operation.compare(AD_DISPLAYED_EVENT) == 0) ||
        (operation.compare(AD_SKIPPED_EVENT) == 0) ||
        (operation.compare(AD_GRATIFIED_EVENT) == 0) ||
        (operation.compare(AD_AVAILABILITY_EVENT) == 0))
    {
        ListenerTargetPairAdEvent* l = _eventMapAdEvent[operation];
        if (l != NULL)
        {
            EVT_SELECTOR_AD_EVENT listener = l->listener;
            PCPokktAdDelegate* target = l->target;
            
            // extract params
            std::map<std::string, std::string> extraValues;
            PCAdConfig* adConfig = fromAdConfigJsonString(param, extraValues);
            (target->*listener)(adConfig, extraValues);
        }
        return;
    }
}

void PCPokktManager::setPokktInitializedListener(EVT_SELECTOR_POKKT_INITIALIZED listener, PCPokktInitializedDelegate* target)
{
    _pokktInitializedListner = listener;
    _pokktInitializedEventTarget = target;
}

void PCPokktManager::removePokktInitializedListener()
{
    _pokktInitializedEventTarget = NULL;
    _pokktInitializedListner = NULL;
}

void PCPokktManager::setOfferwallEventListener(std::string eventName, EVT_SELECTOR_OFFERWALL listener, PCPokktOfferwallDelegate* target)
{
    ListenerTargetPairOfferwall* pair = new ListenerTargetPairOfferwall();
    pair->listener = listener;
    pair->target = target;
    _eventMapOfferwall[eventName] = pair;
}

void PCPokktManager::removeOfferwallEventListener(std::string eventName)
{
    if (_eventMapOfferwall.size() == 0)
        return;
    
    _STL::map<std::string, ListenerTargetPairOfferwall*>::iterator it = _eventMapOfferwall.find(eventName);
    if (it != _eventMapOfferwall.end())
    {
        delete it->second;
        _eventMapOfferwall.erase(it);
    }
}

void PCPokktManager::setAdEventListener(std::string eventName, EVT_SELECTOR_AD_EVENT listener, PCPokktAdDelegate* target)
{
    ListenerTargetPairAdEvent* pair = new ListenerTargetPairAdEvent();
    pair->listener = listener;
    pair->target = target;
    _eventMapAdEvent[eventName] = pair;
}

void PCPokktManager::removeAdEventListener(std::string eventName)
{
    if (_eventMapAdEvent.size() == 0)
        return;
    
    _STL::map<std::string, ListenerTargetPairAdEvent*>::iterator it = _eventMapAdEvent.find(eventName);
    if (it != _eventMapAdEvent.end())
    {
        delete it->second;
        _eventMapAdEvent.erase(it);
    }
}

/**
 * PCParser
 **/
std::string toAdConfigJSONString(PCAdConfig* adConfig)
{
    cJSON* jsonObject = cJSON_CreateObject();
    
    cJSON_AddItemToObject(jsonObject, "screenName", cJSON_CreateString(adConfig->screenName.c_str()));
    cJSON_AddItemToObject(jsonObject, "isRewarded", cJSON_CreateString(adConfig->isRewarded ? "true" : "false"));
    cJSON_AddItemToObject(jsonObject, "shouldAllowSkip", cJSON_CreateString(adConfig->shouldAllowSkip ? "true" : "false"));
    cJSON_AddItemToObject(jsonObject, "defaultSkipTime", cJSON_CreateString(to_string(adConfig->defaultSkipTime).c_str()));
    cJSON_AddItemToObject(jsonObject, "skipConfirmMessage", cJSON_CreateString(adConfig->skipConfirmMessage.c_str()));
    cJSON_AddItemToObject(jsonObject, "backButtonDisabled", cJSON_CreateString(adConfig->backButtonDisabled ? "true" : "false"));
    cJSON_AddItemToObject(jsonObject, "shouldAllowMute", cJSON_CreateString(adConfig->shouldAllowMute ? "true" : "false"));
    cJSON_AddItemToObject(jsonObject, "shouldSkipConfirm", cJSON_CreateString(adConfig->shouldConfirmSkip ? "true" : "false"));
    cJSON_AddItemToObject(jsonObject, "skipConfirmYesLabel", cJSON_CreateString(adConfig->skipConfirmYesLabel.c_str()));
    cJSON_AddItemToObject(jsonObject, "skipConfirmNoLabel", cJSON_CreateString(adConfig->skipConfirmNoLabel.c_str()));
    cJSON_AddItemToObject(jsonObject, "skipTimerMessage", cJSON_CreateString(adConfig->skipTimerMessage.c_str()));
    cJSON_AddItemToObject(jsonObject, "incentiveMessage", cJSON_CreateString(adConfig->incentiveMessage.c_str()));
    
    const char* jsonString = cJSON_PrintUnformatted(jsonObject);
    return jsonString;
}

bool getFinalValueInBool(std::string str)
{
    if(strcmp(str.c_str(), "true") == 0)
    {
        return true;
    }
    return false;
}

PCAdConfig* fromAdConfigJsonString(std::string param, std::map<std::string, std::string>& extraValues)
{
    PCAdConfig* adConfig = new PCAdConfig();
    
    cJSON* jsonObject = cJSON_Parse(param.c_str());
    if (jsonObject == NULL)
        return NULL;
    
    adConfig->screenName = cJSON_GetObjectItem(jsonObject, "screenName")->valuestring;
    adConfig->isRewarded = getFinalValueInBool(cJSON_GetObjectItem(jsonObject, "isRewarded")->valuestring);
    adConfig->backButtonDisabled = getFinalValueInBool(cJSON_GetObjectItem(jsonObject, "backButtonDisabled")->valuestring);
    adConfig->defaultSkipTime = cJSON_GetObjectItem(jsonObject, "defaultSkipTime")->valueint;
    adConfig->shouldAllowSkip = getFinalValueInBool(cJSON_GetObjectItem(jsonObject, "shouldAllowSkip")->valuestring);
    adConfig->shouldAllowMute = getFinalValueInBool(cJSON_GetObjectItem(jsonObject, "shouldAllowMute")->valuestring);
    adConfig->shouldConfirmSkip = getFinalValueInBool(cJSON_GetObjectItem(jsonObject, "shouldSkipConfirm")->valuestring);
    adConfig->skipConfirmMessage = cJSON_GetObjectItem(jsonObject, "skipConfirmMessage")->valuestring;
    adConfig->skipConfirmYesLabel = cJSON_GetObjectItem(jsonObject, "skipConfirmYesLabel")->valuestring;
    adConfig->skipConfirmNoLabel = cJSON_GetObjectItem(jsonObject, "skipConfirmNoLabel")->valuestring;
    adConfig->skipTimerMessage = cJSON_GetObjectItem(jsonObject, "skipTimerMessage")->valuestring;
    adConfig->incentiveMessage = cJSON_GetObjectItem(jsonObject, "incentiveMessage")->valuestring;
   
    // fetch extraValues
    cJSON* jsonObj = cJSON_GetObjectItem(jsonObject, "extraValues");
    if(jsonObj != NULL)
    {
        cJSON* nextExtraValue = cJSON_GetObjectItem(jsonObject, "extraValues")->child;
        while (nextExtraValue != NULL)
        {
            if (nextExtraValue->type == cJSON_String)
            {
                std::string name = nextExtraValue->string;
                std::string element = nextExtraValue->valuestring;
                extraValues[name] = element;
            }
            nextExtraValue = nextExtraValue->next;
        }
    }
    return adConfig;
}

std::string toIAPDetailsJSONString(IAPDetails* details)
{
    cJSON* jsonObject = cJSON_CreateObject();
    cJSON_AddItemToObject(jsonObject, "productId", cJSON_CreateString(details->productId.c_str()));
    cJSON_AddItemToObject(jsonObject, "price", cJSON_CreateString(to_string(details->price).c_str()));
    cJSON_AddItemToObject(jsonObject, "currencyCode", cJSON_CreateString(details->currencyCode.c_str()));
    cJSON_AddItemToObject(jsonObject, "title", cJSON_CreateString(details->title.c_str()));
    cJSON_AddItemToObject(jsonObject, "description", cJSON_CreateString(details->description.c_str()));
    cJSON_AddItemToObject(jsonObject, "purchaseData", cJSON_CreateString(details->purchaseData.c_str()));
    cJSON_AddItemToObject(jsonObject, "purchaseSignature", cJSON_CreateString(details->purchaseSignature.c_str()));
    cJSON_AddItemToObject(jsonObject, "purchaseStore", cJSON_CreateString(details->purchaseStore.c_str()));
    
    const char* jsonString = cJSON_PrintUnformatted(jsonObject);
    
    POKKTLOG("[Pokkt-Marmalade] [toIAPDetailsJSONString] jsonString: %s", jsonString);
    
    return jsonString;
}

std::string toPokktConfigJSONString(PCPokktConfig* config)
{
    cJSON* jsonObject = cJSON_CreateObject();
    
    cJSON_AddItemToObject(jsonObject, "applicationId", cJSON_CreateString(config->applicationId.c_str()));
    cJSON_AddItemToObject(jsonObject, "securityKey", cJSON_CreateString(config->securityKey.c_str()));
    cJSON_AddItemToObject(jsonObject, "integrationType", cJSON_CreateString(config->integrationType.c_str()));
    cJSON_AddItemToObject(jsonObject, "thirdPartyUserId", cJSON_CreateString(config->thirdPartyUserId.c_str()));
    
    cJSON_AddItemToObject(jsonObject, "offerwallAssetValue", cJSON_CreateString(config->offerwallAssetValue.c_str()));
    cJSON_AddItemToObject(jsonObject, "closeOnSuccessFlag", cJSON_CreateString(config->closeOnSuccessFlag ? "true" : "false"));
    
    cJSON_AddItemToObject(jsonObject, "name", cJSON_CreateString(config->name.c_str()));
    cJSON_AddItemToObject(jsonObject, "age", cJSON_CreateString(config->age.c_str()));
    cJSON_AddItemToObject(jsonObject, "sex", cJSON_CreateString(config->sex.c_str()));
    cJSON_AddItemToObject(jsonObject, "mobileNo", cJSON_CreateString(config->mobileNo.c_str()));
    cJSON_AddItemToObject(jsonObject, "emailAddress", cJSON_CreateString(config->emailAddress.c_str()));
    cJSON_AddItemToObject(jsonObject, "location", cJSON_CreateString(config->location.c_str()));
    cJSON_AddItemToObject(jsonObject, "birthday", cJSON_CreateString(config->birthday.c_str()));
    cJSON_AddItemToObject(jsonObject, "maritalStatus", cJSON_CreateString(config->maritalStatus.c_str()));
    cJSON_AddItemToObject(jsonObject, "facebookId", cJSON_CreateString(config->facebookId.c_str()));
    
    cJSON_AddItemToObject(jsonObject, "twitterHandle", cJSON_CreateString(config->twitterHandle.c_str()));
    cJSON_AddItemToObject(jsonObject, "education", cJSON_CreateString(config->education.c_str()));
    cJSON_AddItemToObject(jsonObject, "nationality", cJSON_CreateString(config->nationality.c_str()));
    cJSON_AddItemToObject(jsonObject, "employment", cJSON_CreateString(config->employment.c_str()));
    cJSON_AddItemToObject(jsonObject, "maturityRating", cJSON_CreateString(config->maturityRating.c_str()));

    cJSON_AddItemToObject(jsonObject, "selectedAnalyticsType", cJSON_CreateString(config->selectedAnalyticsType.c_str()));
    cJSON_AddItemToObject(jsonObject, "googleAnalyticsID", cJSON_CreateString(config->googleAnalyticsID.c_str()));
    cJSON_AddItemToObject(jsonObject, "mixPanelProjectToken", cJSON_CreateString(config->mixPanelProjectToken.c_str()));
    cJSON_AddItemToObject(jsonObject, "flurryApplicationKey", cJSON_CreateString(config->flurryApplicationKey.c_str()));
    
    const char* jsonString = cJSON_PrintUnformatted(jsonObject);
    return jsonString;
}
