//
//  AdScreen.cpp
//  PokktSample
//
//  Created by Pankaj Kumar on 22/04/15.
//
//

#include "AdScreen.h"
#include "IwGx.h"
#include "input.h"
#include "resources.h"
#include "UIComponents.h"
#include "PCPokktManager.h"
#include <sstream>
#include "Storage.h"
#include "PCEnums.h"

PCPokktConfig *AdScreen::pokktConfig = new PCPokktConfig();

AdScreen::AdScreen()
{
    POKKTLOG("[POKKT-MARMALADE][AdScreen]video download listener registered");
    
    // listen for pokkt events
    
    PCPokktManager::setAdEventListener(AD_AVAILABILITY_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdAvailability), this);
    PCPokktManager::setAdEventListener(AD_CACHING_COMPLETED_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdCachingCompleted), this);
    PCPokktManager::setAdEventListener(AD_CACHING_FAILED_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdCachingFailed), this);
    PCPokktManager::setAdEventListener(AD_DISPLAYED_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdDisplayed), this);
    PCPokktManager::setAdEventListener(AD_SKIPPED_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdSkipped), this);
    PCPokktManager::setAdEventListener(AD_COMPLETED_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdCompleted), this);
    PCPokktManager::setAdEventListener(AD_CLOSED_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdClosed), this);
    PCPokktManager::setAdEventListener(AD_GRATIFIED_EVENT, PC_AD_EVENT_SELECTOR(AdScreen::handleAdGratified), this);
}

AdScreen::~AdScreen()
{
    PCPokktManager::removeAdEventListener(AD_AVAILABILITY_EVENT);
    PCPokktManager::removeAdEventListener(AD_CACHING_COMPLETED_EVENT);
    PCPokktManager::removeAdEventListener(AD_CACHING_FAILED_EVENT);
    PCPokktManager::removeAdEventListener(AD_DISPLAYED_EVENT);
    PCPokktManager::removeAdEventListener(AD_SKIPPED_EVENT);
    PCPokktManager::removeAdEventListener(AD_COMPLETED_EVENT);
    PCPokktManager::removeAdEventListener(AD_CLOSED_EVENT);
    PCPokktManager::removeAdEventListener(AD_GRATIFIED_EVENT);
}

void AdScreen::handleAdAvailability(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdAvailability] Ad is available to watch");
}

void AdScreen::handleAdCachingCompleted(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdCachingCompleted]");
    
    float adVc = 0;
    std::stringstream(extraValues[AD_VC]) >> adVc;
    
    if(adConfig->isRewarded)
    {
        showButton(showRewardBtn);
        changeButtonLabelTest(showRewardBtnLbl, "Show Reward Ad (Cached)");
    }
    else
    {
        showButton(showNonRewardBtn);
        changeButtonLabelTest(showNonRewardBtnLbl, "Show Non-Reward Ad (Cached)");
    }
}
void AdScreen::handleAdCachingFailed(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdCachingFailed] Ad Caching Failed");
    resetAllButton();
}
void AdScreen::handleAdDisplayed(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdDisplayed]");
    //msgLbl->SetText(message.c_str());
}
void AdScreen::handleAdSkipped(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdSkipped] Ad has been skipped");
}
void AdScreen::handleAdCompleted(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdCompleted] Ad has been watched fully");
}
void AdScreen::handleAdClosed(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdClosed] Ad has been closed");
    resetAllButton();
}

void AdScreen::handleAdGratified(PCAdConfig* adConfig, std::map<std::string, std::string> extraValues)
{
    POKKTLOG("[POKKT-MARMALADE][handleAdGratified] Rewarded successfully after watching Ad %s", extraValues[REWARD].c_str());
    char buf[100] = "";
    sprintf(buf, "%s", extraValues[REWARD].c_str());
    std::string earnedText("Earned points: ");
    std::string earnedPoint(extraValues[REWARD].c_str());
    pointsEarnedLbl->SetText((earnedText + earnedPoint).c_str());
}


void AdScreen::hideButton(CSprite *sprite)
{
    sprite->m_IsVisible = false;
    sprite->m_IsVisible = false;
}

void AdScreen::showButton(CSprite *sprite)
{
    sprite->m_IsVisible = true;
    sprite->m_IsVisible = true;
}

void AdScreen::changeButtonLabelTest(CLabel *label, std::string text)
{
    label->SetText(text);
}

void AdScreen::resetAllButton()
{
    cacheRewardBtn->m_IsVisible = true;
    cacheNonRewardBtn->m_IsVisible = true;
    showRewardBtn->m_IsVisible = true;
    showNonRewardBtn->m_IsVisible = true;
    
    cacheRewardBtnLbl->SetText("Cache Reward Ad");
    cacheNonRewardBtnLbl->SetText("Cache Non-Reward Ad");
    showRewardBtnLbl->SetText("Show Reward Ad");
    showNonRewardBtnLbl->SetText("Show Non-Reward Ad");
}

void AdScreen::Update(float deltaTime, float alphaMul)
{
    if(!m_IsActive)
        return;
    
    Scene:Scene::Update(deltaTime, alphaMul);
    
    //Detect screen tap
    if(m_IsInputActive && m_Manager->GetCurrent() == this && !g_pInput->m_Touched && g_pInput->m_PrevTouched)
    {
        g_pInput->Reset();
        
        if(cacheRewardBtn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            hideButton(cacheRewardBtn);
            hideButton(showRewardBtn);
            
            adConfig = new PCAdConfig();
            adConfig->screenName = screenName;
            adConfig->isRewarded = true;
            PCPokktManager::cacheAd(adConfig);
        }
        else if(cacheNonRewardBtn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            hideButton(cacheNonRewardBtn);
            hideButton(showNonRewardBtn);
            
            adConfig = new PCAdConfig();
            adConfig->screenName = screenName;
            adConfig->isRewarded = false;
            adConfig->shouldAllowMute = false;
            adConfig->shouldAllowSkip = false;
            adConfig->shouldConfirmSkip = false;
            adConfig->defaultSkipTime = 20;
            PCPokktManager::cacheAd(adConfig);
        }
        else if(showRewardBtn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            if(adConfig != NULL)
            {
                PCPokktManager::showAd(adConfig);
            }
            else
            {
                adConfig = new PCAdConfig();
                adConfig->screenName = screenName;
                adConfig->isRewarded = true;
                PCPokktManager::showAd(adConfig);
            }
        }
        else if(showNonRewardBtn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            if(adConfig != NULL)
            {
                adConfig->isRewarded = false;
                PCPokktManager::showAd(adConfig);
            }
            else
            {
                adConfig = new PCAdConfig();
                adConfig->screenName = screenName;
                adConfig->isRewarded = false;
                adConfig->shouldAllowMute = false;
                adConfig->shouldAllowSkip = false;
                adConfig->shouldConfirmSkip = false;
                PCPokktManager::showAd(adConfig);
            }
        }
    }
}

void AdScreen::Render()
{
    Scene::Render();
}

void AdScreen::Init()
{
    Scene::Init();
    
    // Create UI
    CSprite* background = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2, (float)IwGxGetScreenHeight() / 2, g_pResources->getScreenBG(), 0.5f, 0.5f);
    // Fit background to screen size
    background->m_ScaleX = (float)IwGxGetScreenWidth() / background->GetImage()->GetWidth();
    background->m_ScaleY = (float)IwGxGetScreenHeight() / background->GetImage()->GetHeight();
    AddChild(background);
    
    CSprite* logo = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2, (float)IwGxGetScreenHeight() * 0.25f, g_pResources->getLogo(), 0.5f, 0.5f);
    AddChild(logo);
    
    //
    float y_pos = (float)IwGxGetScreenHeight() * 0.45f;
    pointsEarnedLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Points earned: 0");
    AddChild(pointsEarnedLbl);
    
    //button 1
    y_pos = (float)IwGxGetScreenHeight() * 0.56f;
    cacheRewardBtn = UIComponents::getSprite(IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(cacheRewardBtn);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.57f;
    cacheRewardBtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Cache Reward Ad");
    AddChild(cacheRewardBtnLbl);
    
    //button 2
    y_pos = (float)IwGxGetScreenHeight() * 0.64f;
    cacheNonRewardBtn = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(cacheNonRewardBtn);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.65f;
    cacheNonRewardBtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Cache Non-Reward Ad");
    AddChild(cacheNonRewardBtnLbl);
    
    //button3
    y_pos = (float)IwGxGetScreenHeight() * 0.72f;
    showRewardBtn = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(showRewardBtn);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.73f;
    showRewardBtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Show Reward Ad");
    AddChild(showRewardBtnLbl);
    
    //button4
    y_pos = (float)IwGxGetScreenHeight() * 0.80f;
    showNonRewardBtn = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(showNonRewardBtn);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.81f;
    showNonRewardBtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Show Non-Reward Ad");
    AddChild(showNonRewardBtnLbl);
}