//
//  SelectorScreen.cpp
//  PokktSample
//
//  Created by Pankaj Kumar on 22/04/15.
//
//

#include "SelectorScreen.h"
#include "IwGx.h"
#include "input.h"
#include "resources.h"
#include "mainMenu.h"
#include "UIComponents.h"
#include "PCPokktManager.h"
#include "Storage.h"
#include "PCEnums.h"
#include "AdScreen.h"
#include "OfferwallScreen.h"
#include "IAPDetails.h"

PCPokktConfig *SelectorScreen::pokktConfig = new PCPokktConfig();

SelectorScreen::SelectorScreen()
{
    PCPokktManager::setPokktInitializedListener(PC_POKKT_INITIALIZED_EVENT_SELECTOR(SelectorScreen::handlePokktInitialized), this);
}

SelectorScreen::~SelectorScreen() { }

void SelectorScreen::Update(float deltaTime, float alphaMul)
{
    if(!m_IsActive)
        return;
    
    Scene:Scene::Update(deltaTime, alphaMul);
    
    //Detect screen tap
    if(m_IsInputActive && m_Manager->GetCurrent() == this && !g_pInput->m_Touched && g_pInput->m_PrevTouched)
    {
        g_pInput->Reset();
        if(initializeBtn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            initPokkt();
        }
        else if(demoScreen1Btn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            AdScreen* adScreen = (AdScreen*)g_pSceneManager->Find("adscreen");
            adScreen->screenName = "start";
            g_pSceneManager->SwitchTo(adScreen);
        }
        else if(demoScreen2Btn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            AdScreen* adScreen = (AdScreen*)g_pSceneManager->Find("adscreen");
            adScreen->screenName = "levelClear";
            g_pSceneManager->SwitchTo(adScreen);
        }
        else if(offerwallBtn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            OfferwallScreen* offerwallScreen = (OfferwallScreen*)g_pSceneManager->Find("offerwallscreen");
            offerwallScreen->pokktConfig = pokktConfig;
            g_pSceneManager->SwitchTo(offerwallScreen);
        }
        else if(exportLogBtn->HitTest(g_pInput->m_X, g_pInput->m_Y))
        {
            PCPokktManager::exportLog();
        }
    }
}

void SelectorScreen::handlePokktInitialized(bool isReady)
{
    POKKTLOG("[POKKT-CPP][handlePokktInitialized] %d", isReady);
    isPokktReady = isReady;
    if(isReady)
    {
        demoScreen1Btn->m_IsVisible = true;
        demoScreen2Btn->m_IsVisible = true;
        offerwallBtn->m_IsVisible = true;
        
        IAPDetails *iapDetails = new IAPDetails();
        iapDetails->productId = "123";
        iapDetails->price = 100;
        iapDetails->currencyCode = "IN";
        iapDetails->title = "iapDetails";
        iapDetails->description = "description";
        iapDetails->purchaseData = "1-1-2016";
        iapDetails->purchaseSignature = "pankaj";
        iapDetails->purchaseStore = POKKT_IAP_STORE_TYPE_GOOGLE;
        
        POKKTLOG("[Pokkt-Marmalade] Sending trackAPI");
        PCPokktManager::trackIAP(iapDetails);
        PCPokktManager::notifyAppInstall();
    }
}

void SelectorScreen::initPokkt()
{
    PCPokktManager::setDebug(true);
    
    //ios
    pokktConfig->securityKey = "ae501f1003b0dc927dea807498ceaf3c";
    pokktConfig->applicationId = "954b975f9c753bcb8fca42624d78139a";

//    pokktConfig->securityKey = "4a069e89fe4a4e4f58a26f8bff2974ba";
//    pokktConfig->applicationId = "cc41e865391d6ea371b18aff27cc10e6";

    
    //android
//    pokktConfig->securityKey = "iJ02lJss0M";
//    pokktConfig->applicationId = "a2717a45b835b5e9f50284a38d62a74e";
    
//    pokktConfig->securityKey = "91f118562047d230f2b907ba93d2c4e2";
//    pokktConfig->applicationId = "4317fb1fcbf1ca98badf08ef59b0b86b";
    
    pokktConfig->integrationType = POKKT_INTEGRATION_TYPE_ALL;
    
    // pokkt init for video ads
    pokktConfig->thirdPartyUserId = "123456";
    POKKTLOG("[POKKT-CPP][SelectorScreen] Pokkt SDK Version: %s", PCPokktManager::getSDKVersion().c_str());
    pokktConfig->mobileNo = "9900064040";
    pokktConfig->age = "32";
    pokktConfig->birthday = "20/08/1983";
    pokktConfig->education = "MCA";
    pokktConfig->emailAddress = "pankaj454k@gmail.com";
    pokktConfig->employment = "Engineer";
    pokktConfig->facebookId = "pankaj454k";
    pokktConfig->location = "Bangalore";
    pokktConfig->maritalStatus = "Married";
    pokktConfig->maturityRating = "for all";
    pokktConfig->name = "Pankaj";
    pokktConfig->nationality = "Indian";
    pokktConfig->sex = "Male";
    pokktConfig->twitterHandle = "pankaj454k";
    
    //pokktConfig->selectedAnalyticsType = POKKT_ANALYTICS_TYPE_GOOGLE_ANALYTICS;
    //pokktConfig->googleAnalyticsID = "12345";
    
    PCPokktManager::initPokkt(pokktConfig);
}

void SelectorScreen::Render()
{
    Scene::Render();
}

void SelectorScreen::Init()
{
    Scene::Init();
    
    // Create UI
    CSprite* background = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2, (float)IwGxGetScreenHeight() / 2, g_pResources->getScreenBG(), 0.5f, 0.5f);
    // Fit background to screen size
    background->m_ScaleX = (float)IwGxGetScreenWidth() / background->GetImage()->GetWidth();
    background->m_ScaleY = (float)IwGxGetScreenHeight() / background->GetImage()->GetHeight();
    AddChild(background);
    
    CSprite* logo = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2, (float)IwGxGetScreenHeight() * 0.32f, g_pResources->getLogo(), 0.5f, 0.5f);
    AddChild(logo);
    
    
    //Button

    float y_pos = (float)IwGxGetScreenHeight() * 0.56f;
    initializeBtn = UIComponents::getSprite(IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(initializeBtn);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.575f;
    CLabel* initializeBtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Initialize Pokkt");
    AddChild(initializeBtnLbl);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.64f;
    demoScreen1Btn = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(demoScreen1Btn);
    demoScreen1Btn->m_IsVisible = false;
    
    y_pos = (float)IwGxGetScreenHeight() * 0.655f;
    CLabel* demoScreen1BtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Demo Screen 1");
    AddChild(demoScreen1BtnLbl);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.72f;
    demoScreen2Btn = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(demoScreen2Btn);
    demoScreen2Btn->m_IsVisible = false;
    
    y_pos = (float)IwGxGetScreenHeight() * 0.735f;
    CLabel* demoScreen2BtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Demo Screen 2");
    AddChild(demoScreen2BtnLbl);

    
    y_pos = (float)IwGxGetScreenHeight() * 0.80f;
    offerwallBtn = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(offerwallBtn);
    offerwallBtn->m_IsVisible = false;
    
    y_pos = (float)IwGxGetScreenHeight() * 0.815f;
    CLabel* offerwallBtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Demo Offerwall");
    AddChild(offerwallBtnLbl);
    
    
    y_pos = (float)IwGxGetScreenHeight() * 0.88f;
    exportLogBtn = UIComponents::getSprite((float)IwGxGetScreenWidth() / 2.0f, y_pos, g_pResources->getBlueButton(), 0.5f, 0.5f);
    AddChild(exportLogBtn);
    
    y_pos = (float)IwGxGetScreenHeight() * 0.895f;
    CLabel* exportLogBtnLbl = UIComponents::getLabel((float)IwGxGetScreenWidth() / 2.0f, y_pos, (float)IwGxGetScreenWidth(), 30, 0.5f, 0.5f, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP, CColor(0xff, 0xff, 0x30, 0xff), "Export SDK Log");
    AddChild(exportLogBtnLbl);
}
