/*
 * iphone-specific implementation of the PokktNativeExtension extension.
 * Add any platform-specific functionality here.
 */
/*
 * NOTE: This file was originally written by the extension builder, but will not
 * be overwritten (unless --force is specified) and is intended to be modified.
 */
#include "PokktNativeExtension_internal.h"

#import <Foundation/Foundation.h>
#include <string>
#include "IwDebug.h"
#include "s3eDevice.h"
#include "s3eEdk.h"
#include "s3eEdk_iphone.h"

#import <UIKit/UIKit.h>
#import "PokktManager.h"
#import "AdDelegate.h"


@interface PokktAdDelegate:NSObject<AdDelegate>
@end

// Our callback complete function. This will fire once the user has finished responding to
// the MYEXTENSION_CALLBACK_ONEEVENT callback.
static void completeFunction(uint32 extID, int32 notification, void* systemData, void* instance, int32 returnCode, void* completeData)
{
    // We pass the system data as the complete data in s3eEdkCallbacksEnqueue below just for
    // example. Let's sanity check that the callback sent us the same data.
    assert(systemData == completeData);
    
    // Clean up all the memory we allocated. It's safe to do this now as we know the user has
    // finished with the data.
    CallbackData* eventData = (CallbackData*)systemData;
    
    //IwTrace(POKKTNATIVEEXTENSION, ("event data got back as %s and %s ",eventData->operation, eventData->params));
    delete[] eventData->operation;
    delete[] eventData->params;
    delete eventData;
    if (!returnCode) // User indicated everything was okay.
        return;
    // Uh oh! The user indicated something went wrong.
    // Implement logic in your code based on the users response. You might, for example,
    // provide the user with an enum which the can use to return different errors.
    // You can then react based on their response.
}

static void NotifyMarmalade(const char* operation, const char* param)
{
    CallbackData* data = new CallbackData();
    
    const size_t opSize = strlen(operation);
    char* op = new char[opSize+1];
    strncpy(op, operation, opSize);
    op[opSize] = '\0';
    data->operation = op;
    
    const size_t len = strlen(param);
    char* par = new char[len+1];
    strncpy(par, param, len);
    par[len] = '\0';
    data->params = par;
    
    //NSLog(@"NotifyCPP called success. operationString: %s and paramString: %s", data->operation, data->params);
    
    s3eEdkCallbacksEnqueue(S3E_EXT_POKKTNATIVEEXTENSION_HASH, OPERATION_EVENT, data, 0, NULL, S3E_FALSE, completeFunction, data);
}

UIViewController* getRootViewController()
{
    UIViewController *viewController = [[[UIApplication sharedApplication] keyWindow] rootViewController];
    return viewController;
}

NSDictionary* extractJSON(NSString* jsonString)
{
    NSError* jsonError;
    NSData* jsonData = [jsonString dataUsingEncoding:NSUTF8StringEncoding];
    NSDictionary* jsonObject = [NSJSONSerialization JSONObjectWithData:jsonData
                                                               options:NSJSONReadingMutableContainers
                                                                 error:&jsonError];
    return jsonObject;
}

PokktConfig* parsePokktConfig(NSString* configString)
{
    NSDictionary* jsonObject = extractJSON(configString);
    PokktConfig* config = [[PokktConfig alloc] init];
    
    // essential
    config.securityKey = jsonObject[@"securityKey"];
    config.applicationId = jsonObject[@"applicationId"];
    config.thirdPartyUserId = jsonObject[@"thirdPartyUserId"];
    
    // optional
    config.name = jsonObject[@"name"];
    config.age = jsonObject[@"age"];
    config.sex = jsonObject[@"sex"];
    config.mobileNo = jsonObject[@"mobileNo"];
    config.emailAddress = jsonObject[@"emailAddress"];
    config.location = jsonObject[@"location"];
    config.birthday = jsonObject[@"birthday"];
    config.maritalStatus = jsonObject[@"maritalStatus"];
    config.facebookId = jsonObject[@"facebookId"];
    config.twitterHandle = jsonObject[@"twitterHandle"];
    config.education = jsonObject[@"education"];
    config.nationality = jsonObject[@"nationality"];
    config.employment = jsonObject[@"employment"];
    config.maturityRating = jsonObject[@"maturityRating"];
    
    // analytics
    config.googleTrackerID = jsonObject[@"googleAnalyticsID"];
    config.mixPanelTrackerID = jsonObject[@"mixPanelProjectToken"];
    config.flurryTrackerID = jsonObject[@"flurryApplicationKey"];
    
    if(jsonObject[@"selectedAnalyticsType"] != nil && ![jsonObject[@"selectedAnalyticsType"] isEqual:@""])
    {
        if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"FLURRY"])
            config.eventType = FLURRY_ANALYTICS;
        
        else if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"GOOGLE_ANALYTICS"])
            config.eventType = GOOGLE_ANALYTICS;
        
        else if ([jsonObject[@"selectedAnalyticsType"] isEqual:@"MIXPANNEL"])
            config.eventType = MIXPANNEL_ANALYTICS;
    }
    
    return config;
}

AdConfig* parseAdConfig(NSString* configString)
{
    NSDictionary* jsonObject = extractJSON(configString);
    
    AdConfig* adConfig = [[AdConfig alloc] init];
    
    adConfig.screenName = jsonObject[@"screenName"];
    adConfig.isRewarded = [jsonObject[@"isRewarded"] boolValue];
    adConfig.shouldAllowSkip = [jsonObject[@"shouldAllowSkip"] boolValue];
    adConfig.defaultSkipTime = [jsonObject[@"defaultSkipTime"] floatValue];
    adConfig.skipConfirmMessage = jsonObject[@"skipConfirmMessage"];
    adConfig.backButtonDisabled = [jsonObject[@"backButtonDisabled"] boolValue];
    adConfig.shouldAllowMute = [jsonObject[@"shouldAllowMute"] boolValue];
    adConfig.shouldConfirmSkip = [jsonObject[@"shouldSkipConfirm"] boolValue];
    adConfig.skipConfirmYesLabel = jsonObject[@"skipConfirmYesLabel"];
    adConfig.skipConfirmNoLabel = jsonObject[@"skipConfirmNoLabel"];
    adConfig.skipTimerMessage = jsonObject[@"skipTimerMessage"];
    adConfig.incentiveMessage = jsonObject[@"incentiveMessage"];
    
    return adConfig;
}

NSString* stringifyAdConfig(AdConfig* adConfig, NSDictionary* extraValues)
{
    NSMutableDictionary* jsonObject = [[NSMutableDictionary alloc] init];
    
    jsonObject[@"screenName"] = adConfig.screenName;
    jsonObject[@"isRewarded"] = adConfig.isRewarded ? @"true" : @"false";
    jsonObject[@"shouldAllowSkip"] = adConfig.shouldAllowSkip ? @"true" : @"false";
    jsonObject[@"defaultSkipTime"] = [NSNumber numberWithInt:adConfig.defaultSkipTime];
    jsonObject[@"skipConfirmMessage"] = adConfig.skipConfirmMessage;
    jsonObject[@"backButtonDisabled"] = adConfig.backButtonDisabled ? @"true" : @"false";
    jsonObject[@"shouldAllowMute"] = adConfig.shouldAllowMute ? @"true" : @"false";
    jsonObject[@"shouldSkipConfirm"] = adConfig.shouldConfirmSkip ? @"true" : @"false";
    jsonObject[@"skipConfirmYesLabel"] = adConfig.skipConfirmYesLabel;
    jsonObject[@"skipConfirmNoLabel"] = adConfig.skipConfirmNoLabel;
    jsonObject[@"skipTimerMessage"] = adConfig.skipTimerMessage;
    jsonObject[@"incentiveMessage"] = adConfig.incentiveMessage;
    
    if (extraValues != NULL)
        jsonObject[@"extraValues"] = extraValues;
    
    NSError* jsonError;
    NSData* jsonData = [NSJSONSerialization dataWithJSONObject:jsonObject options:0 error:&jsonError];
    NSString* jsonString = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    
    return jsonString;
}

InAppPurchaseRequest* parseIAPDetails(NSString* detailsString)
{
    NSDictionary* jsonObject = extractJSON(detailsString);
    
    InAppPurchaseRequest* detail = [[InAppPurchaseRequest alloc] init];
    
    detail.productIdentifier = jsonObject[@"productId"];
    detail.productPrice = jsonObject[@"price"];
    detail.productCurrency = jsonObject[@"currencyCode"];
    detail.productTitle = jsonObject[@"title"];
    detail.productDescription = jsonObject[@"description"];
    
    // TODO
    //detail.setPurchaseData(jsonObject.optString("purchaseData"));
    //detail.setPurchaseSignature(jsonObject.optString("purchaseSignature"));
    //detail.setPurchaseStore(IAPStoreType.valueOf(jsonObject.optString("purchaseStore")));
    
    return detail;
}

void notifyNative(NSString *operationNSStr, NSString *paramNSStr)
{
    // Pokkt SDK Api
    if ([operationNSStr isEqualToString:@"setDebug"])
    {
        BOOL debug = [paramNSStr isEqualToString:@"true"];
        [PokktManager setDebug:debug];
    }
    
    else if ([operationNSStr isEqualToString:@"exportLog"])
        [PokktManager exportLog:getRootViewController()];
    
    else if ([operationNSStr isEqualToString:@"showToast"])
    {
        NSLog(@"[POKKT-AIR] ShowToast is not supported by the iOS Plugin.");
        [PokktManager showToast:paramNSStr presentingViewController:getRootViewController()];
    }
    
    else if ([operationNSStr isEqualToString:@"initPokkt"])
    {
        NSLog(@"InitPokkt PokktConfig: %@", paramNSStr);
        PokktAdDelegate *delegate = [[PokktAdDelegate alloc] init];
        [PokktManager initPokkt:parsePokktConfig(paramNSStr) withDelegate:delegate];
    }
    
    else if ([operationNSStr isEqualToString:@"updatePokktConfig"])
        [PokktManager updatePokktConfig:parsePokktConfig(paramNSStr)];
    
    // Session Methods
    else if ([operationNSStr isEqualToString:@"startSession"])
        [PokktManager startSession];
    
    else if ([operationNSStr isEqualToString:@"endSession"])
        [PokktManager endSession];
    
    else if ([operationNSStr isEqualToString:@"notifyAppInstall"])
        [PokktManager notifyAppInstall];
    
    else if ([operationNSStr isEqualToString:@"trackIAP"])
        [PokktManager trackIAP:parseIAPDetails(paramNSStr)];
    
    
    // OfferWall Methods -- NOT Supported in iOS Plugin  -- should go to default log case.
    
    
    // Ad Methods
    else if ([operationNSStr isEqualToString:@"cacheAd"])
        [PokktManager cacheAd:parseAdConfig(paramNSStr)];
    
    else if ([operationNSStr isEqualToString:@"showAd"])
        [PokktManager showAd:parseAdConfig(paramNSStr) presentingViewController:getRootViewController()];
    
    else if ([operationNSStr isEqualToString:@"checkAdAvailability"])
        [PokktManager checkAdAvailability:parseAdConfig(paramNSStr)];
    
    else
        NSLog(@"[POKKT-AIR] Unknown operation requested: %@", operationNSStr);
}

//----------------------------START Predefined method------------------------------//
s3eResult PokktNativeExtensionInit_platform()
{
    // Add any platform-specific initialisation code here
    return S3E_RESULT_SUCCESS;
}

void PokktNativeExtensionTerminate_platform()
{
}

void notifyNative_platform(const char* operation, const char* param)
{
    //use this method
    NSString *operationNSStr = [NSString stringWithUTF8String:operation];
    NSString *paramNSStr = [NSString stringWithUTF8String:param];
    notifyNative(operationNSStr, paramNSStr);
}

double getRewardedAdVcOnNative_platform(const char* param)
{
    NSString *paramNSStr = [NSString stringWithUTF8String:(param)];
    return [PokktManager getRewardedAdVc:parseAdConfig(paramNSStr)];
}

const char * getSDKVersionOnNative_platform()
{
    const char* version = [[NSString stringWithFormat:@"%@", [PokktManager getSDKVersion]] UTF8String];
    return version;
}
//----------------------------END Predefined method------------------------------//
    

// events
#define POKKT_INITIALISED_EVENT         "PokktInitialised"
#define AD_CACHING_COMPLETED_EVENT      "AdCachingCompleted"
#define AD_CACHING_FAILED_EVENT         "AdCachingFailed"
#define AD_CLOSED_EVENT                 "AdClosed"
#define AD_COMPLETED_EVENT              "AdCompleted"
#define AD_DISPLAYED_EVENT              "AdDisplayed"
#define AD_SKIPPED_EVENT                "AdSkipped"
#define AD_GRATIFIED_EVENT              "AdGratified"
#define AD_AVAILABILITY_EVENT           "AdAvailability"


@implementation PokktAdDelegate

// use this to notify the framework observer
- (void) notifyFramework:(NSString*) operation param:(NSString*) param
{
    NotifyMarmalade([[NSString stringWithFormat:@"%@", operation] UTF8String], [[NSString stringWithFormat:@"%@", param] UTF8String]);
}

- (void)onPokktInitialised:(BOOL)isReady
{
    [self notifyFramework:@POKKT_INITIALISED_EVENT param:(isReady ? @"true" : @"false")];
}

- (void)onAdCachingCompleted:(AdConfig *)adConfig withReward:(float)reward
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"AD_VC"] = [[NSNumber numberWithFloat:reward] stringValue];
    [self notifyFramework:@AD_CACHING_COMPLETED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdCachingFailed:(AdConfig *)adConfig withErrorMessage:(NSString *)errorMessage
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"MESSAGE"] = errorMessage;
    [self notifyFramework:@AD_CACHING_FAILED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdCompleted:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_COMPLETED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdDisplayed:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_DISPLAYED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdGratified:(AdConfig *)adConfig withReward:(float)reward
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"REWARD"] = [[NSNumber numberWithFloat:reward] stringValue];
    [self notifyFramework:@AD_GRATIFIED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdSkipped:(AdConfig *)adConfig
{
    [self notifyFramework:@AD_SKIPPED_EVENT param:stringifyAdConfig(adConfig, NULL)];
}

- (void)onAdClosed:(AdConfig *)adConfig
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"BACK_BUTTON"] = @"false";
    [self notifyFramework:@AD_CLOSED_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

- (void)onAdAvailabilityStatus:(AdConfig *)adConfig withAvailability:(BOOL)isAdAvailable
{
    NSMutableDictionary* extraValues = [[NSMutableDictionary alloc] init];
    extraValues[@"IS_AVAILABLE"] = (isAdAvailable ? @"true" : @"false");
    [self notifyFramework:@AD_AVAILABILITY_EVENT param:stringifyAdConfig(adConfig, extraValues)];
}

@end
